/*
 * deviced
 *
 * Copyright (c) 2012 - 2015 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdarg.h>
#include "core/log.h"
#include "core/common.h"
#include "apps.h"
#include "core/edbus-handler.h"

#define POPUP_METHOD "PopupLaunch"
#define BUFF_MAX        255

static const struct app_dbus_match {
	const char *type;
	const char *bus;
	const char *path;
	const char *iface;
	const char *method;
} app_match[] = {
	{ APP_DEFAULT , POPUP_BUS_NAME, POPUP_PATH_SYSTEM  , POPUP_INTERFACE_SYSTEM  , POPUP_METHOD },
	{ APP_POWERKEY, POPUP_BUS_NAME, POPUP_PATH_POWERKEY, POPUP_INTERFACE_POWERKEY, POPUP_METHOD },
	{ APP_OVERHEAT, POPUP_BUS_NAME, POPUP_PATH_OVERHEAT, POPUP_INTERFACE_OVERHEAT, POPUP_METHOD },
};

int launch_system_app(char *type, int num, ...)
{
	char *app_type;
	va_list args;
	int i, match, ret;

	if (type)
		app_type = type;
	else
		app_type = APP_DEFAULT;

	match = -1;
	for (i = 0 ; i < ARRAY_SIZE(app_match) ; i++) {
		if (strncmp(app_type, app_match[i].type, strlen(app_type)))
			continue;
		match = i;
		break;
	}
	if (match < 0) {
		_E("Failed to find app matched (%s)", app_type);
		return -EINVAL;
	}

	va_start(args, num);

	ret = dbus_method_async_pairs(app_match[match].bus,
			app_match[match].path,
			app_match[match].iface,
			app_match[match].method,
			num, args);

	va_end(args);
	pm_change_internal(INTERNAL_LOCK_POPUP, LCD_NORMAL);

	return ret;
}

int launch_message_post(char *type)
{
	char *param[1];
	int ret;

	if (!type)
		return -EINVAL;

	param[0] = type;

	ret = dbus_method_async(POPUP_BUS_NAME,
			POPUP_PATH_NOTI,
			POPUP_INTERFACE_NOTI,
			"MessagePostOn",
			"s", param);
	pm_change_internal(INTERNAL_LOCK_POPUP, LCD_NORMAL);
	return ret;
}

int add_async_notification(char *type, dbus_pending_cb func, char *sig, ...)
{
	int ret;
	int count;
	int len;
	char *data = NULL;
	char *param[BUFF_MAX] = {NULL};
	va_list args;

	if (!type)
		return -EINVAL;

	if (sig) {
		len = strlen(sig);
		va_start(args, sig);

		for (count = 0; count < len; count++) {
			data = va_arg(args, char *);
			param[count] = data;
		}

		va_end(args);
	}

	ret = dbus_method_async_with_reply(POPUP_BUS_NAME,
			POPUP_PATH_NOTI,
			POPUP_INTERFACE_NOTI,
			type,
			sig,
			param,
			func,
			-1,
			NULL);

	return ret;
}

int remove_notification(char *type, int id)
{
	char *param[1];
	char id_str[16];

	if (!type || id < 0)
		return -EINVAL;

	snprintf(id_str, sizeof(id_str), "%d", id);
	param[0] = id_str;

	return dbus_method_async(POPUP_BUS_NAME,
			POPUP_PATH_NOTI,
			POPUP_INTERFACE_NOTI,
			type, "i", param);
}
